// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "QuadMesh.hpp"

using namespace Dawn;

void QuadMesh::bufferQuadMeshWithZ(
  Mesh *mesh,
  glm::vec2 xy0, glm::vec2 uv0,
  glm::vec2 xy1, glm::vec2 uv1,
  float_t z, int32_t verticeStart, int32_t indiceStart
) {
  assertNotNull(mesh);

  glm::vec3 positions[QUAD_VERTICE_COUNT] = {
    glm::vec3(xy0, z),
    glm::vec3(xy1.x, xy0.y, z),
    glm::vec3(xy0.x, xy1.y, z),
    glm::vec3(xy1, z)
  };
  glm::vec2 coordinates[QUAD_VERTICE_COUNT] = {
    uv0, glm::vec2(uv1.x, uv0.y),
    glm::vec2(uv0.x, uv1.y), uv1
  };
  meshindice_t indices[QUAD_INDICE_COUNT] = {
    verticeStart, verticeStart + 1, verticeStart + 2,
    verticeStart + 1, verticeStart + 2, verticeStart + 3
  };

  mesh->bufferPositions(verticeStart, positions, QUAD_VERTICE_COUNT);
  mesh->bufferCoordinates(verticeStart, coordinates, QUAD_VERTICE_COUNT);
  mesh->bufferIndices(indiceStart, indices, QUAD_INDICE_COUNT);
}

void QuadMesh::bufferQuadMesh(
  Mesh *mesh,
  glm::vec2 xy0, glm::vec2 uv0,
  glm::vec2 xy1, glm::vec2 uv1,
  int32_t verticeStart, int32_t indiceStart
) {
  QuadMesh::bufferQuadMeshWithZ(
    mesh, xy0, uv0, xy1, uv1, 0, verticeStart, indiceStart
  );
}

void QuadMesh::bufferCoordinates(
  Mesh *mesh,
  glm::vec2 uv0, glm::vec2 uv1,
  int32_t verticeStart
) {
  assertNotNull(mesh);
  glm::vec2 coordinates[QUAD_VERTICE_COUNT] = {
    uv0, glm::vec2(uv1.x, uv0.y),
    glm::vec2(uv0.x, uv1.y), uv1
  };
  mesh->bufferCoordinates(verticeStart, coordinates, QUAD_VERTICE_COUNT);
}

void QuadMesh::bufferPositions(
  Mesh *mesh,
  glm::vec2 xy0, glm::vec2 xy1,
  int32_t verticeStart
) {
  assertNotNull(mesh);
  glm::vec3 positions[QUAD_VERTICE_COUNT] = {
    glm::vec3(xy0, 0),
    glm::vec3(xy1.x, xy0.y, 0),
    glm::vec3(xy0.x, xy1.y, 0),
    glm::vec3(xy1, 0)
  };
  mesh->bufferPositions(verticeStart, positions, QUAD_VERTICE_COUNT);
}

void QuadMesh::initQuadMesh(
  Mesh *mesh,
  glm::vec2 xy0, glm::vec2 uv0,
  glm::vec2 xy1, glm::vec2 uv1,
  float_t z
) {
  assertNotNull(mesh);
  mesh->createBuffers(QUAD_VERTICE_COUNT, QUAD_INDICE_COUNT);
  QuadMesh::bufferQuadMeshWithZ(mesh, xy0, uv0, xy1, uv1, z, 0, 0);
}